<?php
/**
* This file is part of the Agora-Project Software package.
*
* @copyright (c) Agora-Project Limited <https://www.agora-project.net>
* @license GNU General Public License, version 2 (GPL-2.0)
*/


/*
 * Controleur des objets
 */
class CtrlObject extends Ctrl
{
	/*
	 * ACTION : Affiche les logs d'un objet
	 */
	public static function actionLogs()
	{
		if(Req::isParam("targetObjId"))
		{
			$curObj=self::getTargetObj();
			if($curObj->editRight()){
				$vDatas["logsList"]=Db::getTab("SELECT *, UNIX_TIMESTAMP(date) as dateUnix FROM ap_log WHERE objectType='".$curObj::objectType."' AND _idObject=".$curObj->_id." ORDER BY date");
				static::displayPage(Req::commonPath."Vue/VueObjLogs.php",$vDatas);
			}
		}
	}

	/*
	 * ACTION : Telecharge un fichier joint
	 */
	public static function actionDownloadAttachedFile()
	{
		$curFile=MdlObject::getAttachedFile(Req::getParam("_id"));
		if(is_file($curFile["path"]) && $curFile["containerObj"]->isReadable())
			{File::download($curFile["name"],$curFile["path"]);}
	}

	/*
	 * ACTION : Affiche un fichier joint dans le browser
	 */
	 public static function actionDisplayAttachedFile()
	{
		$curFile=MdlObject::getAttachedFile(Req::getParam("_id"));
		if(is_file($curFile["path"]) && $curFile["containerObj"]->isReadable())
			{File::display($curFile["path"]);}
	 }

	/*
	 * ACTION : Supprime un fichier joint
	 */
	public static function actionDeleteAttachedFile()
	{
		$curFile=MdlObject::getAttachedFile(Req::getParam("_id"));
		if(is_file($curFile["path"]) && $curFile["containerObj"]->editRight()){
			$deleteResult=$curFile["containerObj"]->deleteAttachedFile($curFile);
			if($deleteResult==true)  {echo "true";}
		}
	}

	/*
	 * ACTION : Supprime un objet / des objets sélectionnés
	 */
	public static function actionDelete()
	{
		$updateDatasFolderSize=$containerUrl=null;
		foreach(self::getTargetObjects() as $tmpObj){
			if(empty($containerUrl))  {$containerUrl=$tmpObj->getUrl("container");}
			if($tmpObj::moduleName=="file" && $updateDatasFolderSize==null)  {$updateDatasFolderSize=true;}
			$tmpObj->delete();
		}
		if($updateDatasFolderSize==true)	{File::datasFolderSize(true);}//Maj en session
		self::redir($containerUrl);//Redirection sur la page du conteneur de l'objet supprimé
	}

	/*
	 * ACTION : Deplace les objets sélectionnés dans un autre dossier
	 */
	public static function actionFolderMove()
	{
		//Validation du formulaire
		if(Req::isParam("formValidate") && Req::isParam("newFolderId")){
			foreach(self::getTargetObjects() as $tmpObj)  {$tmpObj->folderMove(Req::getParam("newFolderId"));}
			static::lightboxClose();
		}
		//Affiche le menu de déplacement de dossier
		self::folderTree("move");
	}

	/*
	 * VUE : Menu d'arborescence de dossiers ($context: "nav" / "move")
	 */
	public static function folderTree($context="nav")
	{
		$curFolder=self::$curContainer;
		$vDatas["folderTree"]=Ctrl::getObj($curFolder::objectType,1)->folderTree();
		$vDatas["context"]=$context;
		$vueFolderTree=Req::commonPath."Vue/VueObjFolderTree.php";
		if($context=="nav")	{return Ctrl::getVue($vueFolderTree,$vDatas);}//"nav" -> redirige vers un dossier
		else				{static::displayPage($vueFolderTree,$vDatas);}//"move" -> selectionne un dossier pour y déplacer un element
	}

	/*
	 * VUE : Menu du Chemin d'un dossier
	 */
	public static function folderPath()
	{
		if(Ctrl::$curContainer->isRootFolder()==false)	{return Ctrl::getVue(Req::commonPath."Vue/VueObjFolderPath.php");}
	}

	/*
	 * ACTION VUE : Edition d'un dossier
	 */
	public static function actionFolderEdit()
	{
		////	Charge le dossier et Controle d'accès: dossier existant / nouveau dossier
		$curObj=Ctrl::getTargetObj();
		$curObj->controlEdit();
		////	Formulaire validé
		if(Req::isParam("formValidate"))
		{
			//Enregistre et recharge l'objet, puis étend les droits aux sous dossiers?
			$curObj=$curObj->createUpdate("name=".Db::formatParam("name").", description=".Db::formatParam("description"));
			if(Req::isParam("extendToSubfolders")){
				foreach($curObj->folderTree("all") as $tmpObj)	{$tmpObj->setAffectations();}
			}
			static::lightboxClose();
		}
		////	Affiche la vue
		$vDatas["curObj"]=$curObj;
		static::displayPage(Req::commonPath."Vue/VueObjFolderEdit.php",$vDatas);
	}

	/*
	 * AJAX : Controle si un autre objet fichier/dossier porte le même nom (dans le même dossier parent)
	 */
	public static function actionControlDuplicateName()
	{
		$isDuplicate="false";
		if(Req::isParam(["targetObjId","targetObjIdContainer","controledName"]))
		{
			//Charge l'objet conteneur et courant
			$curObj=Ctrl::getTargetObj();
			$curObjContainer=Ctrl::getTargetObj(Req::getParam("targetObjIdContainer"));
			//Recherche les doublons affectés à l'espace courant, dans le dossier courant.. en excluant l'objet courant, si déjà créé
			$sqlSpaceObjects="AND _id IN (select _idObject as _id from ap_objectTarget where objectType='".$curObj::objectType."' and _idSpace=".Ctrl::$curSpace->_id.")";
			$sqlExcludeCurObj=(!empty($curObj->_id))  ?  "AND _id!=".$curObj->_id  :  null;
			$nbDuplicate=Db::getVal("SELECT count(*) FROM ".$curObj::dbTable." WHERE _idContainer=".$curObjContainer->_id." ".$sqlSpaceObjects." ".$sqlExcludeCurObj." AND name=".Db::formatParam("controledName"));
			if($nbDuplicate>0)   {$isDuplicate="true";}
		}
		echo $isDuplicate;
	}

	/*
	 * AJAX : Controle si tous les sous-dossiers peuvent être supprimés
	 */
	public static function actionSubFoldersDeleteControl()
	{
		$curFolder=Ctrl::getTargetObj();
		$folderTreeFull=$curFolder->FolderTree("all");
		$folderTreeReadAccess=$curFolder->FolderTree(2);
		echo (count($folderTreeFull)==count($folderTreeReadAccess)) ? "true" : "false";
	}
	
	/*
	 * AJAX : Edition d'un dossier => Control le droit d'accès d'un user/espace au dossier parent
	 */
	public static function actionAccessRightParentFolder()
	{
		//Init
		$parentFolder=Ctrl::getTargetObj();
		$objectRight=explode("_",Req::getParam("objectRight"));//exple: "1_U2_2" ou "1_spaceUsers_1.5"
		//Controle?
		if($parentFolder->isRootFolder()==false && count($objectRight)==3)
		{
			$sqlObjSelect="SELECT count(*) FROM ap_objectTarget WHERE objectType=".Db::format($parentFolder::objectType)." AND _idObject=".(int)$parentFolder->_id." AND _idSpace=".(int)$objectRight[0];
			$sqlTargetSelect=($objectRight[1]=="spaceUsers")  ?  " AND target='spaceUsers'"  :  " AND (target='spaceUsers' OR target=".Db::format($objectRight[1]).")";
			if(Db::getVal($sqlObjSelect.$sqlTargetSelect)==0)
			{
				$ajaxResult["error"]=true;
				if(preg_match("/^G/i",$objectRight[1]))		{$targetLabel=self::getObj("userGroup",str_ireplace("G","",$objectRight[1]))->title;}
				elseif(preg_match("/^U/i",$objectRight[1]))	{$targetLabel=self::getObj("user",str_ireplace("U","",$objectRight[1]))->display();}
				else										{$targetLabel=Txt::trad("EDIT_allUsers");}
				$ajaxResult["message"]=str_replace(["--TARGET_LABEL--","--FOLDER_NAME--"], [$targetLabel,Txt::reduce($parentFolder->name,30)], Txt::trad("EDIT_parentFolderAccessError"));
				echo json_encode($ajaxResult);
			}
		}
	}

	/*
	 * AJAX : Valide une évaluation Like
	 */
	public static function actionUsersLikeValidate()
	{
		//Vérifs de base
		if(Ctrl::$curUser->isUser() && Req::isParam("targetObjId"))
		{
			//Init
			$curObj=self::getTargetObj();
			//Applique la nouvelle valeur / le changement de valeur
			$newValue=(Req::getParam("likeValue")=="like")  ?  1  :  0;
			$sqlValueUser="WHERE objectType='".$curObj::objectType."' AND _idObject=".$curObj->_id." AND _idUser=".Ctrl::$curUser->_id;
			$oldValue=Db::getVal("SELECT value FROM ap_objectLike ".$sqlValueUser);			//recup l'ancienne valeur
			if($oldValue!=null)  {Db::query("DELETE FROM ap_objectLike ".$sqlValueUser);}	//reinit la valeur?
			if($oldValue==null || $newValue!=$oldValue)  {Db::query("INSERT INTO ap_objectLike SET objectType='".$curObj::objectType."', _idObject=".$curObj->_id.", _idUser=".Ctrl::$curUser->_id.", value=".$newValue);}//Ajoute la nouvelle valeur si elle change
			//Nb et liste des personnes qui likes / dontlike
			$usersLikeList=$usersDontlikeList=null;
			foreach(array_merge($curObj->getUsersLike("like"),$curObj->getUsersLike("dontlike"))  as $tmpLike){
				$tmpUserName=Ctrl::getObj("user",$tmpLike["_idUser"])->display().", ";
				if(!empty($tmpLike["value"]))  {$usersLikeList.=$tmpUserName;}  else  {$usersDontlikeList.=$tmpUserName;}
			}
			$ajaxResult["usersLikeList"]=trim($usersLikeList, ", ");
			$ajaxResult["usersDontlikeList"]=trim($usersDontlikeList, ", ");
			$ajaxResult["nbLikes"]=count($curObj->getUsersLike("like"));
			$ajaxResult["nbDontlikes"]=count($curObj->getUsersLike("dontlike"));
			echo json_encode($ajaxResult);
		}
	}

	/*
	 * ACTION : Affiche les commentaires d'un objet
	 */
	public static function actionComments()
	{
		////	Charge l'element
		$curObj=Ctrl::getTargetObj();
		$curObj->controlRead();
		////	Ajoute / Modif / Supprime un commentaire
		if(Req::isParam(["formValidate","comment"]) && Req::getParam("actionComment")=="add")
			{Db::query("INSERT INTO ap_objectComment SET objectType='".$curObj::objectType."', _idObject=".$curObj->_id.", _idUser=".self::$curUser->_id.", dateCrea=".Db::dateNow().", comment=".Db::formatParam("comment"));}
		elseif(Req::isParam("idComment") && MdlObjectAttributes::userCommentEditRight(Req::getParam("idComment"))){
			$sqlSelectComment="_id=".Db::formatParam("idComment")." AND objectType='".$curObj::objectType."' AND _idObject=".$curObj->_id;
			if(Req::getParam("actionComment")=="delete")	{Db::query("DELETE FROM ap_objectComment WHERE ".$sqlSelectComment);}
			elseif(Req::getParam("actionComment")=="modif")	{Db::query("UPDATE ap_objectComment SET comment=".Db::formatParam("comment")." WHERE ".$sqlSelectComment);}
		}
		////	Affiche la vue
		$vDatas["curObj"]=$curObj;
		$vDatas["updateCircleNb"]=Req::isParam("actionComment");
		$vDatas["commentList"]=Db::getTab("SELECT * FROM ap_objectComment WHERE objectType='".$curObj::objectType."' AND _idObject=".$curObj->_id." ORDER BY dateCrea DESC");
		static::displayPage(Req::commonPath."Vue/VueObjComments.php",$vDatas);
	}

	/*
	 * ACTION : formulaire de recherche
	 */
	public static function actionSearch()
	{
		$vDatas=array();
		//Champs de recherche de tous les objets
		$vDatas["searchFields"]=array();
		foreach(self::$curSpace->moduleList() as $tmpModule)
		{
			if(isset($tmpModule["ctrl"]::$MdlObjects))
			{
				//Pour chaque objet du module, on liste les champs de recherche : modifie le "title" et le "checked"
				foreach($tmpModule["ctrl"]::$MdlObjects as $tmpMdlObject)
				{
					foreach($tmpMdlObject::$searchFields as $tmpField)
					{
						$vDatas["searchFields"][$tmpField]["checked"]=(!Req::isParam("searchFields") || in_array($tmpField,Req::getParam("searchFields"))) ? "checked" : "";
						if(empty($vDatas["searchFields"][$tmpField]["title"]))	{$vDatas["searchFields"][$tmpField]["title"]="";}
						$folderInTitle=preg_match("/".Txt::trad("objectFolder")."/i",$vDatas["searchFields"][$tmpField]["title"]);
						if($tmpMdlObject::isFolder==true && $folderInTitle==false)	{$vDatas["searchFields"][$tmpField]["title"].="- ".Txt::trad("OBJECTfolder")."<br>";}
						elseif($tmpMdlObject::isFolder==false)						{$vDatas["searchFields"][$tmpField]["title"].="- ".Txt::trad("OBJECT".$tmpMdlObject::objectType)."<br>";}
					}
				}
			}
		}
		//Resultat de recherche
		if(Req::isParam("formValidate"))
		{
			//prépare la recherche
			$pluginParams=array(
				"type"=>"search",
				"searchText"=>Txt::clean(Req::getParam("searchText"),"normal"," "),
				"searchMode"=>Req::getParam("searchMode"),
				"searchFields"=>Req::getParam("searchFields"),
				"creationDate"=>Req::getParam("creationDate"),
				"searchModules"=>Req::getParam("searchModules")
			);
			//récupère les résultats!
			$vDatas["pluginsSearchResult"]=array();
			foreach(self::$curSpace->moduleList() as $tmpModule){
				if(in_array($tmpModule["ctrl"]::moduleName,Req::getParam("searchModules")) && method_exists($tmpModule["ctrl"],"plugin")){
					$vDatas["pluginsSearchResult"]=array_merge($vDatas["pluginsSearchResult"],$tmpModule["ctrl"]::plugin($pluginParams));
				}
			}
		}
		static::displayPage(Req::commonPath."/Vue/VueSearch.php",$vDatas);
	}
}